@@ -2,13 +2,13 @@ from collections.abc import Iterable
 import numpy as np
 import warnings
 
-from hdmf.utils import docval, getargs, popargs, call_docval_func, get_docval
+from hdmf.common import DynamicTable, DynamicTableRegion
+from hdmf.utils import docval, popargs, get_docval, get_data_shape, popargs_to_dict
 
 from . import register_class, CORE_NAMESPACE
 from .base import TimeSeries
 from .image import ImageSeries
 from .core import NWBContainer, MultiContainerInterface, NWBDataInterface
-from hdmf.common import DynamicTable, DynamicTableRegion
 from .device import Device
 
 
@@ -21,10 +21,10 @@ class OpticalChannel(NWBContainer):
 
     @docval({'name': 'name', 'type': str, 'doc': 'the name of this electrode'},  # required
             {'name': 'description', 'type': str, 'doc': 'Any notes or comments about the channel.'},  # required
-            {'name': 'emission_lambda', 'type': 'float', 'doc': 'Emission wavelength for channel, in nm.'})  # required
+            {'name': 'emission_lambda', 'type': float, 'doc': 'Emission wavelength for channel, in nm.'})  # required
     def __init__(self, **kwargs):
         description, emission_lambda = popargs("description", "emission_lambda", kwargs)
-        call_docval_func(super(OpticalChannel, self).__init__, kwargs)
+        super().__init__(**kwargs)
         self.description = description
         self.emission_lambda = emission_lambda
 
@@ -50,17 +50,17 @@ class ImagingPlane(NWBContainer):
              'doc': 'One of possibly many groups storing channel-specific data.'},
             {'name': 'description', 'type': str, 'doc': 'Description of this ImagingPlane.'},  # required
             {'name': 'device', 'type': Device, 'doc': 'the device that was used to record'},  # required
-            {'name': 'excitation_lambda', 'type': 'float', 'doc': 'Excitation wavelength in nm.'},  # required
+            {'name': 'excitation_lambda', 'type': float, 'doc': 'Excitation wavelength in nm.'},  # required
             {'name': 'indicator', 'type': str, 'doc': 'Calcium indicator'},  # required
             {'name': 'location', 'type': str, 'doc': 'Location of image plane.'},  # required
-            {'name': 'imaging_rate', 'type': 'float',
+            {'name': 'imaging_rate', 'type': float,
              'doc': 'Rate images are acquired, in Hz. If the corresponding TimeSeries is present, the rate should be '
                     'stored there instead.', 'default': None},
             {'name': 'manifold', 'type': 'array_data',
              'doc': ('DEPRECATED: Physical position of each pixel. size=("height", "width", "xyz"). '
                      'Deprecated in favor of origin_coords and grid_spacing.'),
              'default': None},
-            {'name': 'conversion', 'type': 'float',
+            {'name': 'conversion', 'type': float,
              'doc': ('DEPRECATED: Multiplier to get from stored values to specified unit (e.g., 1e-3 for millimeters) '
                      'Deprecated in favor of origin_coords and grid_spacing.'),
              'default': 1.0},
@@ -87,38 +87,118 @@ class ImagingPlane(NWBContainer):
              'doc': "Measurement units for grid_spacing. The default value is 'meters'.",
              'default': 'meters'})
     def __init__(self, **kwargs):
-        optical_channel, description, device, excitation_lambda, imaging_rate, \
-            indicator, location, manifold, conversion, unit, reference_frame, origin_coords, origin_coords_unit, \
-            grid_spacing, grid_spacing_unit = popargs(
-                'optical_channel', 'description', 'device', 'excitation_lambda',
-                'imaging_rate', 'indicator', 'location', 'manifold', 'conversion',
-                'unit', 'reference_frame', 'origin_coords', 'origin_coords_unit', 'grid_spacing', 'grid_spacing_unit',
-                kwargs)
-        call_docval_func(super(ImagingPlane, self).__init__, kwargs)
-        self.optical_channel = optical_channel if isinstance(optical_channel, list) else [optical_channel]
-        self.description = description
-        self.device = device
-        self.excitation_lambda = excitation_lambda
-        self.imaging_rate = imaging_rate
-        self.indicator = indicator
-        self.location = location
-        if manifold is not None:
+        keys_to_set = ('optical_channel',
+                       'description',
+                       'device',
+                       'excitation_lambda',
+                       'imaging_rate',
+                       'indicator',
+                       'location',
+                       'manifold',
+                       'conversion',
+                       'unit',
+                       'reference_frame',
+                       'origin_coords',
+                       'origin_coords_unit',
+                       'grid_spacing',
+                       'grid_spacing_unit')
+        args_to_set = popargs_to_dict(keys_to_set, kwargs)
+        super().__init__(**kwargs)
+
+        if not isinstance(args_to_set['optical_channel'], list):
+            args_to_set['optical_channel'] = [args_to_set['optical_channel']]
+        if args_to_set['manifold'] is not None:
             warnings.warn("The 'manifold' argument is deprecated in favor of 'origin_coords' and 'grid_spacing'.",
                           DeprecationWarning)
-        if conversion != 1.0:
+        if args_to_set['conversion'] != 1.0:
             warnings.warn("The 'conversion' argument is deprecated in favor of 'origin_coords' and 'grid_spacing'.",
                           DeprecationWarning)
-        if unit != 'meters':
+        if args_to_set['unit'] != 'meters':
             warnings.warn("The 'unit' argument is deprecated in favor of 'origin_coords_unit' and 'grid_spacing_unit'.",
                           DeprecationWarning)
-        self.manifold = manifold
-        self.conversion = conversion
-        self.unit = unit
-        self.reference_frame = reference_frame
-        self.origin_coords = origin_coords
-        self.origin_coords_unit = origin_coords_unit
-        self.grid_spacing = grid_spacing
-        self.grid_spacing_unit = grid_spacing_unit
+        for key, val in args_to_set.items():
+            setattr(self, key, val)
+
+
+@register_class("OnePhotonSeries", CORE_NAMESPACE)
+class OnePhotonSeries(ImageSeries):
+    """Image stack recorded over time from 1-photon microscope."""
+
+    __nwbfields__ = (
+        "imaging_plane", "pmt_gain", "scan_line_rate", "exposure_time", "binning", "power", "intensity"
+    )
+
+    @docval(
+        *get_docval(ImageSeries.__init__, "name"),  # required
+        {"name": "imaging_plane", "type": ImagingPlane, "doc": "Imaging plane class/pointer."},  # required
+        *get_docval(ImageSeries.__init__, "data", "unit", "format"),
+        {"name": "pmt_gain", "type": float, "doc": "Photomultiplier gain.", "default": None},
+        {
+            "name": "scan_line_rate",
+            "type": float,
+            "doc": (
+                "Lines imaged per second. This is also stored in /general/optophysiology but is kept "
+                "here as it is useful information for analysis, and so good to be stored w/ the actual data."
+             ),
+            "default": None,
+        },
+        {
+            "name": "exposure_time",
+            "type": float,
+            "doc": "Exposure time of the sample; often the inverse of the frequency.",
+            "default": None,
+        },
+        {
+            "name": "binning",
+            "type": (int, "uint"),
+            "doc": "Amount of pixels combined into 'bins'; could be 1, 2, 4, 8, etc.",
+            "default": None,
+        },
+        {
+            "name": "power",
+            "type": float,
+            "doc": "Power of the excitation in mW, if known.",
+            "default": None,
+        },
+        {
+            "name": "intensity",
+            "type": float,
+            "doc": "Intensity of the excitation in mW/mm^2, if known.",
+            "default": None,
+        },
+        *get_docval(
+            ImageSeries.__init__,
+            "external_file",
+            "starting_frame",
+            "bits_per_pixel",
+            "dimension",
+            "resolution",
+            "conversion",
+            "timestamps",
+            "starting_time",
+            "rate",
+            "comments",
+            "description",
+            "control",
+            "control_description",
+            "device",
+            "offset",
+        )
+    )
+    def __init__(self, **kwargs):
+        keys_to_set = (
+            "imaging_plane", "pmt_gain", "scan_line_rate", "exposure_time", "binning", "power", "intensity"
+        )
+        args_to_set = popargs_to_dict(keys_to_set, kwargs)
+        super().__init__(**kwargs)
+
+        if args_to_set["binning"] is not None and args_to_set["binning"] < 0:
+            raise ValueError(f"Binning value must be >= 0: {args_to_set['binning']}")
+        if isinstance(args_to_set["binning"], int):
+            args_to_set["binning"] = np.uint(args_to_set["binning"])
+
+        for key, val in args_to_set.items():
+            setattr(self, key, val)
 
 
 @register_class('TwoPhotonSeries', CORE_NAMESPACE)
@@ -132,28 +212,27 @@ class TwoPhotonSeries(ImageSeries):
 
     @docval(*get_docval(ImageSeries.__init__, 'name'),  # required
             {'name': 'imaging_plane', 'type': ImagingPlane, 'doc': 'Imaging plane class/pointer.'},  # required
-            {'name': 'data', 'type': ('array_data', 'data', TimeSeries), 'shape': ([None] * 3, [None] * 4),
-             'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames',
-             'default': None},
-            *get_docval(ImageSeries.__init__, 'unit', 'format'),
+            *get_docval(ImageSeries.__init__, 'data', 'unit', 'format'),
             {'name': 'field_of_view', 'type': (Iterable, TimeSeries), 'shape': ((2, ), (3, )),
              'doc': 'Width, height and depth of image, or imaged area (meters).', 'default': None},
-            {'name': 'pmt_gain', 'type': 'float', 'doc': 'Photomultiplier gain.', 'default': None},
-            {'name': 'scan_line_rate', 'type': 'float',
+            {'name': 'pmt_gain', 'type': float, 'doc': 'Photomultiplier gain.', 'default': None},
+            {'name': 'scan_line_rate', 'type': float,
              'doc': 'Lines imaged per second. This is also stored in /general/optophysiology but is kept '
                     'here as it is useful information for analysis, and so good to be stored w/ the actual data.',
              'default': None},
             *get_docval(ImageSeries.__init__, 'external_file', 'starting_frame', 'bits_per_pixel',
                         'dimension', 'resolution', 'conversion', 'timestamps', 'starting_time', 'rate',
-                        'comments', 'description', 'control', 'control_description'))
+                        'comments', 'description', 'control', 'control_description', 'device', 'offset'))
     def __init__(self, **kwargs):
-        field_of_view, imaging_plane, pmt_gain, scan_line_rate = popargs(
-            'field_of_view', 'imaging_plane', 'pmt_gain', 'scan_line_rate', kwargs)
-        call_docval_func(super(TwoPhotonSeries, self).__init__, kwargs)
-        self.field_of_view = field_of_view
-        self.imaging_plane = imaging_plane
-        self.pmt_gain = pmt_gain
-        self.scan_line_rate = scan_line_rate
+        keys_to_set = ("field_of_view",
+                       "imaging_plane",
+                       "pmt_gain",
+                       "scan_line_rate")
+        args_to_set = popargs_to_dict(keys_to_set, kwargs)
+        super().__init__(**kwargs)
+
+        for key, val in args_to_set.items():
+            setattr(self, key, val)
 
 
 @register_class('CorrectedImageStack', CORE_NAMESPACE)
@@ -164,22 +243,22 @@ class CorrectedImageStack(NWBDataInterface):
     assumed to be 2-D (has only x & y dimensions).
     """
 
-    __nwbfields__ = ({'name': 'corrected', 'child': True},
-                     {'name': 'xy_translation', 'child': True},
+    __nwbfields__ = ({'name': 'corrected', 'child': True, 'required_name': 'corrected'},
+                     {'name': 'xy_translation', 'child': True, 'required_name': 'xy_translation'},
                      'original')
 
     @docval({'name': 'name', 'type': str,
              'doc': 'The name of this CorrectedImageStack container', 'default': 'CorrectedImageStack'},
             {'name': 'corrected', 'type': ImageSeries,
-             'doc': 'Image stack with frames shifted to the common coordinates.'},
+             'doc': 'Image stack with frames shifted to the common coordinates. This must have the name "corrected".'},
             {'name': 'original', 'type': ImageSeries,
              'doc': 'Link to image series that is being registered.'},
             {'name': 'xy_translation', 'type': TimeSeries,
              'doc': 'Stores the x,y delta necessary to align each frame to the common coordinates, '
-                    'for example, to align each frame to a reference image.'})
+                    'for example, to align each frame to a reference image. This must have the name "xy_translation".'})
     def __init__(self, **kwargs):
         corrected, original, xy_translation = popargs('corrected', 'original', 'xy_translation', kwargs)
-        call_docval_func(super(CorrectedImageStack, self).__init__, kwargs)
+        super().__init__(**kwargs)
         self.corrected = corrected
         self.original = original
         self.xy_translation = xy_translation
@@ -196,7 +275,7 @@ class MotionCorrection(MultiContainerInterface):
         'get': 'get_corrected_image_stack',
         'create': 'create_corrected_image_stack',
         'type': CorrectedImageStack,
-        'attr': 'corrected_images_stacks'
+        'attr': 'corrected_image_stacks'
     }
 
 
@@ -231,10 +310,9 @@ class PlaneSegmentation(DynamicTable):
             *get_docval(DynamicTable.__init__, 'id', 'columns', 'colnames'))
     def __init__(self, **kwargs):
         imaging_plane, reference_images = popargs('imaging_plane', 'reference_images', kwargs)
-        if kwargs.get('name') is None:
+        if kwargs['name'] is None:
             kwargs['name'] = imaging_plane.name
-        columns, colnames = getargs('columns', 'colnames', kwargs)
-        call_docval_func(super(PlaneSegmentation, self).__init__, kwargs)
+        super().__init__(**kwargs)
         self.imaging_plane = imaging_plane
         if isinstance(reference_images, ImageSeries):
             reference_images = (reference_images,)
@@ -263,15 +341,15 @@ class PlaneSegmentation(DynamicTable):
             rkwargs['pixel_mask'] = pixel_mask
         if voxel_mask is not None:
             rkwargs['voxel_mask'] = voxel_mask
-        return super(PlaneSegmentation, self).add_row(**rkwargs)
+        return super().add_row(**rkwargs)
 
     @staticmethod
     def pixel_to_image(pixel_mask):
         """Converts a 2D pixel_mask of a ROI into an image_mask."""
         image_matrix = np.zeros(np.shape(pixel_mask))
         npmask = np.asarray(pixel_mask)
-        x_coords = npmask[:, 0].astype(np.int)
-        y_coords = npmask[:, 1].astype(np.int)
+        x_coords = npmask[:, 0].astype(np.int32)
+        y_coords = npmask[:, 1].astype(np.int32)
         weights = npmask[:, -1]
         image_matrix[y_coords, x_coords] = weights
         return image_matrix
@@ -294,7 +372,7 @@ class PlaneSegmentation(DynamicTable):
             {'name': 'region', 'type': (slice, list, tuple), 'doc': 'the indices of the table', 'default': slice(None)},
             {'name': 'name', 'type': str, 'doc': 'the name of the ROITableRegion', 'default': 'rois'})
     def create_roi_table_region(self, **kwargs):
-        return call_docval_func(self.create_region, kwargs)
+        return self.create_region(**kwargs)
 
 
 @register_class('ImageSegmentation', CORE_NAMESPACE)
@@ -336,15 +414,37 @@ class RoiResponseSeries(TimeSeries):
     @docval(*get_docval(TimeSeries.__init__, 'name'),  # required
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),  # required
              'shape': ((None, ), (None, None)),
-             'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
+             'doc': ('The data values. May be 1D or 2D. The first dimension must be time. The optional second '
+                     'dimension represents ROIs')},
             *get_docval(TimeSeries.__init__, 'unit'),
             {'name': 'rois', 'type': DynamicTableRegion,  # required
              'doc': 'a table region corresponding to the ROIs that were used to generate this data'},
             *get_docval(TimeSeries.__init__, 'resolution', 'conversion', 'timestamps', 'starting_time', 'rate',
-                        'comments', 'description', 'control', 'control_description'))
+                        'comments', 'description', 'control', 'control_description', 'offset'))
     def __init__(self, **kwargs):
         rois = popargs('rois', kwargs)
-        call_docval_func(super(RoiResponseSeries, self).__init__, kwargs)
+
+        data_shape = get_data_shape(data=kwargs["data"], strict_no_data_load=True)
+        rois_shape = get_data_shape(data=rois.data, strict_no_data_load=True)
+        if (
+            data_shape is not None and rois_shape is not None
+
+            # check that data is 2d and rois is 1d
+            and len(data_shape) == 2 and len(rois_shape) == 1
+
+            # check that key dimensions are known
+            and data_shape[1] is not None and rois_shape[0] is not None
+
+            and data_shape[1] != rois_shape[0]
+        ):
+            if data_shape[0] == rois_shape[0]:
+                warnings.warn("%s '%s': The second dimension of data does not match the length of rois, "
+                              "but instead the first does. Data is oriented incorrectly and should be transposed."
+                              % (self.__class__.__name__, kwargs["name"]))
+            else:
+                warnings.warn("%s '%s': The second dimension of data does not match the length of rois. "
+                              "Your data may be transposed." % (self.__class__.__name__, kwargs["name"]))
+        super().__init__(**kwargs)
         self.rois = rois
 
 
