@@ -7,10 +7,11 @@ from hdmf.backends.hdf5 import HDF5IO
 from hdmf.common import DynamicTable
 
 from pynwb import NWBFile, TimeSeries, NWBHDF5IO, get_manager
+from pynwb.base import Image, Images
 from pynwb.file import Subject
 from pynwb.epoch import TimeIntervals
 from pynwb.ecephys import ElectricalSeries
-from pynwb.testing import NWBH5IOMixin, TestCase, remove_test_file
+from pynwb.testing import NWBH5IOMixin, NWBH5IOFlexMixin, TestCase, remove_test_file
 
 
 class TestNWBFileHDF5IO(TestCase):
@@ -203,14 +204,43 @@ class TestSubjectIO(NWBH5IOMixin, TestCase):
 
     def setUpContainer(self):
         """ Return the test Subject """
-        return Subject(age='12 mo',
-                       description='An unfortunate rat',
-                       genotype='WT',
-                       sex='M',
-                       species='Rattus norvegicus',
-                       subject_id='RAT123',
-                       weight='2 lbs',
-                       date_of_birth=datetime(1970, 1, 1, 12, tzinfo=tzutc()))
+        return Subject(
+            age="P90D",
+            age__reference="gestational",
+            description="An unfortunate rat",
+            genotype="WT",
+            sex="M",
+            species="Rattus norvegicus",
+            subject_id="RAT123",
+            weight="2 kg",
+            date_of_birth=datetime(1970, 1, 1, 12, tzinfo=tzutc()),
+            strain="my_strain",
+        )
+
+    def addContainer(self, nwbfile):
+        """ Add the test Subject to the given NWBFile """
+        nwbfile.subject = self.container
+
+    def getContainer(self, nwbfile):
+        """ Return the test Subject from the given NWBFile """
+        return nwbfile.subject
+
+
+class TestSubjectAgeReferenceNotSetIO(NWBH5IOMixin, TestCase):
+
+    def setUpContainer(self):
+        """ Return the test Subject """
+        return Subject(
+            age="P90D",
+            description="An unfortunate rat",
+            genotype="WT",
+            sex="M",
+            species="Rattus norvegicus",
+            subject_id="RAT123",
+            weight="2 kg",
+            date_of_birth=datetime(1970, 1, 1, 12, tzinfo=tzutc()),
+            strain="my_strain",
+        )
 
     def addContainer(self, nwbfile):
         """ Add the test Subject to the given NWBFile """
@@ -237,7 +267,7 @@ class TestEpochsIO(NWBH5IOMixin, TestCase):
         """ Add the test epochs to the given NWBFile """
         nwbfile.add_epoch_column(
             name='temperature',
-            description='average temperture (c) during epoch'
+            description='average temperature (c) during epoch'
         )
 
         nwbfile.add_epoch(
@@ -310,7 +340,7 @@ class TestEpochsIODf(TestEpochsIO):
                                [(4, 1, tsa)]],
                 'tags': [[''], [''], ['fizz', 'buzz'], ['qaz']]
             },
-            index=pd.Index(np.arange(4), name='id')
+            index=pd.Index(np.arange(4, dtype=np.int64), name='id')
         )
         # pop the timeseries column out because ts_obt has rows of lists of tuples and ts_exp has rows of lists of lists
         ts_obt = df_obt.pop('timeseries')
@@ -337,7 +367,7 @@ class TestEpochsIODf(TestEpochsIO):
                 'stop_time': [0.25, 0.30, 0.40, 0.45],
                 'tags': [[''], [''], ['fizz', 'buzz'], ['qaz']]
             },
-            index=pd.Index(np.arange(4), name='id')
+            index=pd.Index(np.arange(4, dtype=np.int64), name='id')
         )
 
         df_obt = self.read_container.to_dataframe(exclude=set(['timeseries', 'timeseries_index']))
@@ -439,22 +469,91 @@ class TestElectrodes(NWBH5IOMixin, TestCase):
         """
         Return placeholder table for electrodes. Tested electrodes are added directly to the NWBFile in addContainer
         """
-        return DynamicTable('electrodes', 'a placeholder table')
+        return DynamicTable(name='electrodes', description='a placeholder table')
 
     def addContainer(self, nwbfile):
         """ Add electrodes and related objects to the given NWBFile """
         self.dev1 = nwbfile.create_device(name='dev1')
-        self.group = nwbfile.create_electrode_group(name='tetrode1', description='tetrode description',
-                                                    location='tetrode location', device=self.dev1)
+        self.group = nwbfile.create_electrode_group(
+            name='tetrode1',
+            description='tetrode description',
+            location='tetrode location',
+            device=self.dev1
+        )
 
-        nwbfile.add_electrode(id=1, x=1.0, y=2.0, z=3.0, imp=-1.0, location='CA1', filtering='none', group=self.group,
-                              group_name='tetrode1')
-        nwbfile.add_electrode(id=2, x=1.0, y=2.0, z=3.0, imp=-2.0, location='CA1', filtering='none', group=self.group,
-                              group_name='tetrode1')
-        nwbfile.add_electrode(id=3, x=1.0, y=2.0, z=3.0, imp=-3.0, location='CA1', filtering='none', group=self.group,
-                              group_name='tetrode1')
-        nwbfile.add_electrode(id=4, x=1.0, y=2.0, z=3.0, imp=-4.0, location='CA1', filtering='none', group=self.group,
-                              group_name='tetrode1')
+        nwbfile.add_electrode(
+            id=1,
+            x=1.0, y=2.0, z=3.0,
+            imp=-1.0,
+            location='CA1',
+            filtering='none',
+            group=self.group,
+            group_name='tetrode1'
+        )
+        nwbfile.add_electrode(
+            id=2,
+            x=1.0, y=2.0, z=3.0,
+            imp=-2.0,
+            location='CA1',
+            filtering='none',
+            group=self.group,
+            group_name='tetrode1'
+        )
+
+        self.container = nwbfile.electrodes  # override self.container which has the placeholder
+
+    def getContainer(self, nwbfile):
+        """ Return the test electrodes table from the given NWBFile """
+        return nwbfile.electrodes
+
+    def test_roundtrip(self):
+        super().test_roundtrip()
+        # When comparing the pandas dataframes for the row we drop the 'group' column since the
+        # ElectrodeGroup object after reading will naturally have a different address
+        pd.testing.assert_frame_equal(self.read_container[0].drop('group', axis=1),
+                                      self.container[0].drop('group', axis=1))
+
+
+class TestElectrodesOptColumns(NWBH5IOMixin, TestCase):
+
+    def setUpContainer(self):
+        """
+        Return placeholder table for electrodes. Tested electrodes are added directly to the NWBFile in addContainer
+        """
+        return DynamicTable(name='electrodes', description='a placeholder table')
+
+    def addContainer(self, nwbfile):
+        """ Add electrodes and related objects to the given NWBFile """
+        self.dev1 = nwbfile.create_device(name='dev1')
+        self.group = nwbfile.create_electrode_group(
+            name='tetrode1',
+            description='tetrode description',
+            location='tetrode location',
+            device=self.dev1
+        )
+
+        nwbfile.add_electrode(
+            id=1,
+            x=1.0, y=2.0, z=3.0,
+            imp=-1.0,
+            location='CA1',
+            filtering='none',
+            group=self.group,
+            group_name='tetrode1',
+            rel_x=4.0, rel_y=5.0, rel_z=6.0,
+            reference='ref1'
+        )
+        nwbfile.add_electrode(
+            id=2,
+            x=1.0, y=2.0, z=3.0,
+            imp=-2.0,
+            location='CA1',
+            filtering='none',
+            group=self.group,
+            group_name='tetrode1',
+            rel_x=4.0, rel_y=5.0, rel_z=6.0,
+            reference='ref1'
+        )
 
         self.container = nwbfile.electrodes  # override self.container which has the placeholder
 
@@ -476,7 +575,7 @@ class TestElectrodesRegion(NWBH5IOMixin, TestCase):
         """
         Return placeholder table for electrodes. Tested electrodes are added directly to the NWBFile in addContainer
         """
-        return DynamicTable('electrodes', 'a placeholder table')
+        return DynamicTable(name='electrodes', description='a placeholder table')
 
     def addContainer(self, nwbfile):
         """ Add electrode table region and related objects to the given NWBFile """
@@ -516,3 +615,35 @@ class TestElectrodesRegion(NWBH5IOMixin, TestCase):
         super().test_roundtrip()
         for ii, item in enumerate(self.read_container):
             pd.testing.assert_frame_equal(self.table[ii+1], item)
+
+
+class TestAddStimulusTemplateTimeSeries(NWBH5IOFlexMixin, TestCase):
+
+    def getContainerType(self):
+        return "time series stored as a stimulus template"
+
+    def addContainer(self):
+        ts = TimeSeries(
+            name="test_ts",
+            data=[0, 1, 2, 3, 4, 5],
+            unit="grams",
+            timestamps=[0.0, 0.1, 0.2, 0.3, 0.4, 0.5]
+        )
+        self.nwbfile.add_stimulus_template(ts)
+
+    def getContainer(self, nwbfile):
+        return nwbfile.get_stimulus_template("test_ts")
+
+
+class TestAddStimulusTemplateImages(NWBH5IOFlexMixin, TestCase):
+
+    def getContainerType(self):
+        return "images stored as a stimulus template"
+
+    def addContainer(self):
+        image1 = Image(name="test_image1", data=np.ones((10, 10)))
+        images = Images(name="images_name", images=[image1])
+        self.nwbfile.add_stimulus_template(images)
+
+    def getContainer(self, nwbfile):
+        return nwbfile.get_stimulus_template("images_name")
